module MyHCalc where 

--type Parse b a = [b] -> [ (a, [b]) ] 
type Parse symbol result = [symbol] -> [(result, [symbol])] 
data Expr = Lit Int | Bin Op Expr Expr              deriving (Eq,Show)
data Op   = Add | Sub | Mul | Div | Mod             deriving (Eq,Show)

succeed :: b -> Parse a b
succeed val inp = [(val,inp)]

-- symbola "abc" -- [('a',"bc")]
-- symbola "bcd" -- []
symbola :: Parse Char Char
symbola []     = []      
symbola (x:xs)
     | 'a'==x    = [ ('a',xs) ]
     | otherwise = []


-- symbol 'a' "abc" -- [('a',"bc")]
-- symbol 'a' "bcd" -- []
symbol :: Eq a => a -> Parse a a
-- symbol t = spot (==t)
symbol s []      = []	     
symbol s (x:xs)
    | s==x      = [ (x,xs) ]
    | otherwise = []

-- spot (=='(') "(1+2)" -- [('(',"1+2)")]
-- spot (=='(') "1+2"
spot :: (a-> Bool) -> Parse a a
spot f []       = []
spot f (x:xs)
    | f x       = [(x,xs)]
    | otherwise = []

-- dig "1+2" -- [('1',"+2")]
dig      :: Parse Char Char 
dig input = (spot isDigit) input

-- Wende Parser p1 und p2 auf den Input an
-- (dig <|> symbol 'a') "abc" -- [('a',"bc")]
-- (dig <|> symbol 'a') "123" -- [('1',"23")]
infixr 4 <|>
(<|>)             :: Parse a b -> Parse a b -> Parse a b
(p1 <|> p2) input  = (p1 input) ++ (p2 input)

-- Sequenzierung: 
-- (symbol 'a' <*> symbol 'b' <*> symbol 'c') "abc" -- [(('a',('b','c')),"")]
-- (symbol 'a' <*> symbol 'b') "abc" -- [(('a','b'),"c")]
-- (symbol 'a' <*> symbol 'b') "123" -- []
infixr 6 <*> -- legt Rechtsassoziativitaet und Prioritaet fest.
(<*>)             :: Parse a b -> Parse a c -> Parse a (b,c)
(p1 <*> p2) input  = [( (x1, x2), xs2 ) 
		      | (x1, xs1) <- p1 input
		     ,  (x2, xs2) <- p2 xs1 
		     ]

-- Wende eine Funktion f auf den Input an und das Ergebnis Parser
infixr 5 <@
(<@)           :: Parse a b -> ( b -> c ) -> Parse a c
(p <@ f) input  = [ (f x,xs) 
                    | (x,xs) <- p input 
                  ]

-- many dig "12A" --[("12","A"),("1","2A"),("","12A")]
-- many dig ""  -- [("","")]
-- many (spot isAlpha) "123" -- [("","123")]
-- many (spot isAlpha) "test" -- [("test",""),("tes","t"),("te","st"),("t","est"),("","test")]
many   :: Parse a b -> Parse a [b]
many p  = (p <*> many p <@ list) 
	  <|> 
	  (succeed [])
    where list (x,xs) = x:xs


-- many1 dig "" -- []
many1   :: Parse a b -> Parse a [b]
many1 p = p <*> many p <@ list
    where list (x,xs) = x:xs

-- option (symbol '-') "-123" -- [("-","123"),("","-123")]
-- option (symbol '-') "123" -- [("","123")]
option         :: Parse a b -> Parse a [b]
option p input  = ( ( p <@ (:[]) ) 
		  <|> 
		    ( succeed [] )) input
--------------------------------------------------------------------

isOp :: Char -> Bool
isOp o = (o == '+') || (o=='-') || (o == '*') || (o=='/') || (o == '%')

charToOp :: Char -> Op
charToOp  c 
    = case c of
      '+' -> Add
      '-' -> Sub
      '*' -> Mul
      '/' -> Div
      '%' -> Mod	

-- parser "(1+11)"
-- parser "(25+-11)"
-- parser "(2*2)"
parser :: Parse Char Expr
parser  = litParse <|> opExprParse 

-- [ (('(',(Lit 12,('+',(Lit 12,')')))),"")]
opExprParse :: Parse Char Expr
opExprParse  = (symbol '('<*>
		parser    <*>
		spot isOp <*>
		parser    <*>
		symbol ')'
	       ) <@ makeExpr
    where 
    makeExpr :: (a,(Expr,(Char,(Expr,b)))) -> Expr
    makeExpr (_,(e1,(bop,(e2,_))))  = Bin (charToOp bop) e1 e2

-- litParse "12"  -- [(Lit 12,""),(Lit 1,"2")]
-- litParse "-12" -- [(Lit (-12),""),(Lit (-1),"2")]
litParse :: Parse  Char Expr
litParse 
    = (
       option (symbol '-') <*>  many1 (spot isDigit) 
      ) <@  charListToExpr.join 
    where join = uncurry (++)

-- charlistToExpr "12" -- Lit 12
-- charlistToExpr "-12" -- Lit (-12)
charListToExpr :: [Char] -> Expr
charListToExpr []     = Lit 0
charListToExpr (c:xs) = Lit ( natural (c:xs) )

-- wandelt eine Liste von Ziffern in ein Int um
natural :: [Char] -> Int
natural []       = 0
natural ('-':xs) =  (-1) * natural xs 
natural (c:xs)   = toDigit c *10^(length xs) + natural xs

-- wandelt eine Ziffer vom Typ Char in ein Int um.       
toDigit :: Char -> Int
toDigit c
    | isDigit c  = ord c - ord '0'
    | otherwise  = 0

-- top-level parser
topLevel :: Parse a b -> [a] -> b
topLevel p inp
    = case results of
      [] -> error "Fehler im Ausdruck!"
      _  -> head results  
    where
    results = [ found | (found,[]) <- p inp ]
----------------------------------------------------------------
-- Auswertung des Expression

opValue :: Op -> Int -> Int -> Int
opValue Add e1 e2 = e1   +   e2
opValue Sub e1 e2 = e1   -   e2 
opValue Mul e1 e2 = e1   *   e2 
opValue Div e1 e2 = e1 `div` e2 
opValue Mod e1 e2 = e1 `mod` e2

eval :: Expr -> Int
eval (Lit n) = n
eval (Bin op e1 e2)   = (opValue op v1 v2)
    where 
    v1 = eval e1 
    v2 = eval e2 

-- Hauptfunktion
-- calc "(15+-3)"
calc :: String -> String
calc str = str ++ " = " ++ show (eval ((topLevel parser) str) )
---------------------------------------------------------------
-- epsilon :: Parse [a] [a]
-- epsilon  = succeed []
{-
digit = spot isDigit <@ f
	where f c = ord c - ord '0'

-- ( natural1 <*> symbol '(' ) "12(" -- [((12,'('),"")]
natural1 :: Parse Char Int
natural1  = many digit <@ foldl f 0
    where f a b = a *10+b




endOfInput :: b -> Parse a b
endOfInput x [] = [(x,[])]
endOfInput x _  = []



--epsilon inp = Nil
data Tree = Nil
	  | Bin(Tree, Tree)   deriving (Eq,Show)

parens :: Parse Char Tree
parens  =  ((symbol '(' 
	    <*> parens 
            <*> symbol ')' 
	    <*> parens 
	    ) <@ makeTree) 
	    <|> succeed Nil 
	    where makeTree (_,(x,(_,y)))  =  Bin(x,y)  




space :: Parse Char Char -> Parse Char Char
space inp = [ (x,xs) |
	      (x,xs) <- (spot (==' ')) inp
	    ]
	      
	


       

--sat :: (Char -> Bool)->Char
--sat p = do{c <-item; if p c then return c else zero}

--space :: Parser()
--space  = do{ many(sat isSpace) 
--	     return ()
--	   }
-}
















