module MyPCalc2 where

import Parsec
import ParsecExpr
import qualified ParsecToken as PT
import qualified ParsecLanguage as PL
import MyGeneral

lexer :: PT.TokenParser ()
lexer  = PT.makeTokenParser( PL.haskellDef
			    { PL.commentStart      = ""
                            , PL.commentEnd        = ""
                            , PL.commentLine       = ""
                            , PL.nestedComments    = True
                            , PL.identStart        = letter <|> char '_'      
                            , PL.identLetter       = alphaNum <|> oneOf "_"
                            , PL.opLetter          = oneOf "*/+-"
			    , PL.reservedOpNames= ["*","/","+","-"]
                            , PL.reservedNames  = []
                            , PL.caseSensitive  = True
			    })

 			 

whiteSpace= PT.whiteSpace lexer
lexeme    = PT.lexeme     lexer
symbol    = PT.symbol     lexer
natural   = PT.natural    lexer
parens    = PT.parens     lexer
semi      = PT.semi       lexer
identifier= PT.identifier lexer
reserved  = PT.reserved   lexer
reservedOp= PT.reservedOp lexer

expr :: Parser Integer
expr  = buildExpressionParser table factor
	<?> "expression"

table = [
         [prefix  "-"  negate, prefix "+" id],
         [postfix "++" (+1)],
         [op "*" (*) AssocLeft, op "/" div AssocLeft],
         [op "+" (+) AssocLeft, op "-" (-) AssocLeft]
	]
    where
    op      name fun assoc = Infix  (do{ reservedOp name; return fun <?> "operator"}) assoc
    prefix  name fun       = Prefix (do{ reservedOp name; return fun <?> "prefix"  }) 
    postfix name fun       = Postfix(do{ reservedOp name; return fun
         <?> "postfix"              })         
       


factor = parens expr
	 <|> natural
	 <?> "simple expression" 

runLex :: Show a => Parser a -> String -> IO()
runLex p input 
    = run (
	   do { whiteSpace
	      ; x <- p
	      ; eof
	      ; return x
	      }
	  ) input


{-
 runLex expr "1 + 2"
 runLex expr "1+  2"
-} 



